@@ -195,11 +195,6 @@ public:												// public methods
     // receives the control message. the data written to the message created
     // by *NewControlMessage() can be read directly from m.
 
-    // shall the server accept sync messages from the clients?
-    virtual bool AcceptClientSync() const	{
-        return false;
-    }
-
     // con/desstruction
     eTeam();											// default constructor
     eTeam(nMessage &m);									// remote constructor
