@@ -256,6 +256,7 @@ public:
 
     // shall the server accept sync messages from the clients?
     virtual bool AcceptClientSync() const;
+    static bool AcceptClientSyncStatic();
 
 
     void GetID();			// get a network ID
@@ -349,6 +350,13 @@ template<class T> class nNOInitialisator:public nDescriptor{
             }
             else
             {
+                if (sn_GetNetState()==nSERVER && !T::AcceptClientSyncStatic())
+                {
+                    // client is not allowed to send objects of this type, kick and ignore
+                    Cheater(m.SenderID());
+                    return;
+                }
+
                 nNetObjectRegistrar registrar;
                 //			nNetObject::RegisterRegistrar( registrar );
                 tJUST_CONTROLLED_PTR< T > n=new T(m);
@@ -366,13 +374,15 @@ template<class T> class nNOInitialisator:public nDescriptor{
 
                 if (sn_GetNetState()==nSERVER && !n->AcceptClientSync())
                 {
-                    Cheater(m.SenderID()); // cheater!
-                    n->Release();
-                }
-                else if ( static_cast< nNetObject* >( sn_netObjects[ n->ID() ] ) != n )
-                {
-                    // object was unable to be registered
-                    n->Release(); // silently delete it.
+#ifdef DEBUG
+                    tERR_WARN("AcceptClientSync was supposed to be checked earler.");
+#endif
+                    Cheater(m.SenderID());
+
+                    // deregister
+                    if(sn_netObjects[ n->ID() ].operator->() == n.operator->() )
+                        sn_netObjects[ n->ID() ] = NULL;
+                    n = NULL;
                 }
             }
 #ifndef NOEXCEPT
