@@ -28,6 +28,8 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "gServerBrowser.h"
 #include "gGame.h"
 #include "gLogo.h"
+#include "gServerFavorites.h"
+#include "gFriends.h"
 
 #include "nServerInfo.h"
 #include "nNetwork.h"
@@ -38,20 +40,28 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "rRender.h"
 
 #include "uMenu.h"
+#include "uInputQueue.h"
 
 #include "tMemManager.h"
 #include "tSysTime.h"
+#include "tToDo.h"
 
 #include "tDirectories.h"
+#include "tConfiguration.h"
 
 int gServerBrowser::lowPort  = 4534;
 
 int gServerBrowser::highPort = 4540;
 static bool continuePoll = false;
-static const int sg_simultaneous = 5;
+static int sg_simultaneous = 20;
+static tSettingItem< int > sg_simultaneousConf( "BROWSER_QUERIES_SIMULTANEOUS", sg_simultaneous );
 
 static tOutput *sg_StartHelpText = NULL;
 
+nServerInfo::QueryType sg_queryType = nServerInfo::QUERY_OPTOUT;
+tCONFIG_ENUM( nServerInfo::QueryType );
+static tSettingItem< nServerInfo::QueryType > sg_query_type( "BROWSER_QUERY_FILTER", sg_queryType );
+
 class gServerMenuItem;
 
 
@@ -59,23 +69,35 @@ class gServerInfo: public nServerInfo
 {
 public:
     gServerMenuItem *menuItem;
+	bool show; //for server browser hiding
 
-    gServerInfo():menuItem(NULL)
+    gServerInfo():menuItem(NULL), show(true)
     {
-    };
+    }
 
     virtual ~gServerInfo();
+
+    // during browsing, the whole server list consists of gServerInfos
+    static gServerInfo * GetFirstServer()
+    {
+        return dynamic_cast< gServerInfo * >( nServerInfo::GetFirstServer() );
+    }
+
+    gServerInfo * Next()
+    {
+        return dynamic_cast< gServerInfo * >( nServerInfo::Next() );
+    }
 };
 
 nServerInfo* CreateGServer()
 {
     nServerInfo *ret = tNEW(gServerInfo);
 
-    if (!continuePoll)
-    {
-        nServerInfo::StartQueryAll();
-        continuePoll = true;
-    }
+    //if (!continuePoll)
+    //{
+    //    nServerInfo::StartQueryAll( sg_queryType );
+    //    continuePoll = true;
+    // }
 
     return ret;
 }
@@ -83,10 +105,24 @@ nServerInfo* CreateGServer()
 
 class gServerMenu: public uMenu
 {
+    int sortKey_;
+
 public:
+    virtual void OnRender();
+
     void Update(); // sort the server view by score
     gServerMenu(const char *title);
     ~gServerMenu();
+
+    virtual void HandleEvent( SDL_Event event );
+
+    void Render(REAL y,
+                const tString &servername, const tOutput &score,
+                const tOutput &users     , const tOutput &ping);
+
+    void Render(REAL y,
+                const tString &servername, const tString &score,
+                const tString &users     , const tString &ping);
 };
 
 
@@ -97,6 +133,9 @@ protected:
     {
     }
 
+    // handles a key press
+    virtual bool Event( SDL_Event& event );
+
     virtual void RenderBackground();
 };
 
@@ -104,8 +143,10 @@ class gServerMenuItem: public gBrowserMenuItem
 {
 protected:
     gServerInfo *server;
-
+    double      lastPing_; //!< the time of the last manual ping
+    bool        favorite_; //!< flag indicating whether this is a favorite
 public:
+    void AddFavorite();
     void SetServer(nServerInfo *s);
     gServerInfo *GetServer();
 
@@ -114,6 +155,9 @@ public:
 
     virtual void Enter();
 
+    // handles a key press
+    virtual bool Event( SDL_Event& event );
+
     gServerMenuItem(gServerMenu *men);
     virtual ~gServerMenuItem();
 };
@@ -138,6 +182,21 @@ static bool sg_RequestLANcontinuously = false;
 
 void gServerBrowser::BrowseMaster()
 {
+    BrowseSpecialMaster(0,"");
+}
+
+// the currently active master
+static nServerInfoBase * sg_currentMaster = 0;
+nServerInfoBase * gServerBrowser::CurrentMaster()
+{
+    return sg_currentMaster;
+}
+
+
+void gServerBrowser::BrowseSpecialMaster( nServerInfoBase * master, char const * prefix )
+{
+    sg_currentMaster = master;
+
     sg_RequestLANcontinuously = false;
 
     sn_ServerInfoCreator *cback = nServerInfo::SetCreator(&CreateGServer);
@@ -146,17 +205,17 @@ void gServerBrowser::BrowseMaster()
     sr_con.fullscreen=true;
 
 #ifndef DEDICATED
-    sr_SwapGL();
-    sr_ClearGL();
-    sr_SwapGL();
-    sr_ClearGL();
+    rSysDep::SwapGL();
+    rSysDep::ClearGL();
+    rSysDep::SwapGL();
+    rSysDep::ClearGL();
 #endif
 
     bool to=sr_textOut;
     sr_textOut=true;
 
     nServerInfo::DeleteAll();
-    nServerInfo::GetFromMaster();
+    nServerInfo::GetFromMaster( master, prefix );
     nServerInfo::Save();
 
     //  gLogo::SetBig(true);
@@ -170,11 +229,14 @@ void gServerBrowser::BrowseMaster()
 
     BrowseServers();
 
+    nServerInfo::Save();
+
     sg_TalkToMaster = false;
 
     nServerInfo::SetCreator(cback);
-}
 
+    sg_currentMaster = master;
+}
 
 void gServerBrowser::BrowseLAN()
 {
@@ -188,10 +250,10 @@ void gServerBrowser::BrowseLAN()
     sr_con.fullscreen=true;
 
 #ifndef DEDICATED
-    sr_SwapGL();
-    sr_ClearGL();
-    sr_SwapGL();
-    sr_ClearGL();
+    rSysDep::SwapGL();
+    rSysDep::ClearGL();
+    rSysDep::SwapGL();
+    rSysDep::ClearGL();
 #endif
 
     bool to=sr_textOut;
@@ -213,8 +275,10 @@ void gServerBrowser::BrowseLAN()
 
 void gServerBrowser::BrowseServers()
 {
-    nServerInfo::StartQueryAll();
-    continuePoll = nServerInfo::DoQueryAll(sg_simultaneous);
+    //nServerInfo::CalcScoreAll();
+    //nServerInfo::Sort();
+    nServerInfo::StartQueryAll( sg_queryType );
+    continuePoll = true;
 
     gServerMenu browser("Server Browser");
 
@@ -229,6 +293,12 @@ void gServerBrowser::BrowseServers()
       ConnectToServer(nServerInfo::GetFirstServer());
     */
     browser.Update();
+
+    // eat excess input the user made while the list was fetched
+    SDL_Event ignore;
+    REAL time;
+    while(su_GetSDLInput(ignore, time)) ;
+
     browser.Enter();
 
     nServerInfo::GetFromLANContinuouslyStop();
@@ -242,35 +312,113 @@ void gServerBrowser::BrowseServers()
 
 
 
+void gServerMenu::HandleEvent( SDL_Event event )
+{
+#ifndef DEDICATED
+    switch (event.type)
+    {
+    case SDL_KEYDOWN:
+        switch (event.key.keysym.sym)
+        {
+        case(SDLK_LEFT):
+            sortKey_ = ( sortKey_ + nServerInfo::KEY_MAX-1 ) % nServerInfo::KEY_MAX;
+            Update();
+            return;
+            break;
+        case(SDLK_RIGHT):
+            sortKey_ = ( sortKey_ + 1 ) % nServerInfo::KEY_MAX;
+            Update();
+            return;
+            break;
+		case(SDLK_m):
+			FriendsToggle();
+            Update();
+			return;
+			break;
+        default:
+            break;
+        }
+    }
+#endif
 
+    uMenu::HandleEvent( event );
+}
+
+void gServerMenu::OnRender()
+{
+    uMenu::OnRender();
+
+    // next time the server list is to be resorted
+    static double sg_serverMenuRefreshTimeout=-1E+32f;
+
+    if (sg_serverMenuRefreshTimeout < tSysTimeFloat())
+    {
+        Update();
+        sg_serverMenuRefreshTimeout = tSysTimeFloat()+2.0f;
+    }
+}
 
 void gServerMenu::Update()
 {
     // get currently selected server
     gServerMenuItem *item = NULL;
     if ( selected < items.Len() )
+    {
         item = dynamic_cast<gServerMenuItem*>(items(selected));
+    }
     gServerInfo* info = NULL;
     if ( item )
     {
         info = item->GetServer();
     }
 
+    // keep the cursor position relative to the top, if possible
+    int selectedFromTop = items.Len() - selected;
+
     ReverseItems();
 
-    nServerInfo::Sort();
+    nServerInfo::CalcScoreAll();
+    nServerInfo::Sort( nServerInfo::PrimaryKey( sortKey_ ) );
 
     int mi = 1;
-    nServerInfo *run = nServerInfo::GetFirstServer();
+    gServerInfo *run = gServerInfo::GetFirstServer();
+	bool oneFound = false; //so we can display all if none were found
     while (run)
     {
-        if (mi >= items.Len())
-            tNEW(gServerMenuItem)(this);
-
-        gServerMenuItem *item = dynamic_cast<gServerMenuItem*>(items(mi));
-        item->SetServer(run);
+		//check friend filter
+		if (getFriendsEnabled())
+		{
+			run->show = false;
+			int i;
+			tString userNames = run->UserNames();
+			tString* friends = getFriends();
+			for (i = MAX_FRIENDS-1; i>=0; i--)
+			{
+				if (run->Users() > 0 && friends[i].Len() > 1 && userNames.StrPos(friends[i]) >= 0)
+				{
+					oneFound = true;
+					run->show = true;
+				}
+			}
+		}
         run = run->Next();
-        mi ++;
+	}
+
+	run = gServerInfo::GetFirstServer();
+	{
+   		while (run)
+    	{
+			if (run->show || oneFound == false)
+			{
+	        	if (mi >= items.Len())
+    		        tNEW(gServerMenuItem)(this);
+
+    	    	gServerMenuItem *item = dynamic_cast<gServerMenuItem*>(items(mi));
+    	    	item->SetServer(run);
+	    	    mi++;
+			}
+        	run = run->Next();
+		}
     }
 
     if (items.Len() == 1)
@@ -284,8 +432,11 @@ void gServerMenu::Update()
 
     ReverseItems();
 
-    // set cursor to currently selected server
-    if ( info )
+    // keep the cursor position relative to the top, if possible ( calling function will handle the clamping )
+    selected = items.Len() - selectedFromTop;
+
+    // set cursor to currently selected server, if possible
+    if ( info && info->menuItem )
     {
         selected = info->menuItem->GetID();
     }
@@ -299,7 +450,7 @@ void gServerMenu::Update()
             nServerInfo::GetFromLANContinuously();
             if (!continuePoll)
             {
-                nServerInfo::StartQueryAll();
+                nServerInfo::StartQueryAll( sg_queryType );
                 continuePoll = true;
             }
             timeout = tSysTimeFloat()+10;
@@ -309,6 +460,7 @@ void gServerMenu::Update()
 
 gServerMenu::gServerMenu(const char *title)
         : uMenu(title, false)
+        , sortKey_( nServerInfo::KEY_SCORE )
 {
     nServerInfo *run = nServerInfo::GetFirstServer();
     while (run)
@@ -342,9 +494,9 @@ static REAL text_width=.025;
 static REAL shrink = .6f;
 static REAL displace = .15;
 
-static void Render(REAL y,
-                   const tString &servername, const tString &score,
-                   const tString &users     , const tString &ping)
+void gServerMenu::Render(REAL y,
+                         const tString &servername, const tString &score,
+                         const tString &users     , const tString &ping)
 {
     if (sr_glOut)
     {
@@ -355,41 +507,34 @@ static void Render(REAL y,
 
 
         //int posDisplacement = 0;
-        tString text;
-        if (servername.Len() > 1)
+        tColoredString text;
+        if (tColoredString::RemoveColors(servername).Len() > 1)
         {
-            /*	for (int g=0; g<strlen(servername); g++)
-            	{
-            		if (servername[g] == '0' && servername[g+1] == 'x')
-            		{
-            			posDisplacement+=8;
-            			g+=7;
-            		}
-            	}
-            	if (posDisplacement > 0)
-            		text << servername << ColorString(1,1,1);
-            	else*/
-            text << RemoveColors(servername);
-            //text.SetPos(strlen(servername)+posDisplacement, false);
+            text << servername << "0xRESETT";
         }
         else
             text << tOutput("$network_master_unknown");
 
         if (ping.Len() > 1)
         {
-            text.SetPos(static_cast<int>(1.35/c.GetCWidth())  - ping.Len(), false );
-            text << ping;
+            text.SetPos(static_cast<int>(1.35/c.GetCWidth())  - tColoredString::RemoveColors( ping ).Len() - 1, true );
+            text << "0xRESETT";
+            text << " " << ping;
+        }
+        else
+        {
+            text << "0xRESETT";
         }
 
         if (users.Len() > 1)
         {
-            text.SetPos(static_cast<int>(1.6/c.GetCWidth()) - users.Len(), false );
+            text.SetPos(static_cast<int>(1.6/c.GetCWidth()) - tColoredString::RemoveColors( users ).Len(), false );
             text << users;
         }
 
         if (score.Len() > 1)
         {
-            text.SetPos(static_cast<int>(1.8/c.GetCWidth()) - score.Len(), false );
+            text.SetPos(static_cast<int>(1.8/c.GetCWidth()) - tColoredString::RemoveColors( score ).Len(), false );
             text << score;
         }
 
@@ -397,33 +542,65 @@ static void Render(REAL y,
     }
 }
 
-static void Render(REAL y,
-                   const tOutput &servername, const tOutput &score,
-                   const tOutput &users     , const tOutput &ping)
+void gServerMenu::Render(REAL y,
+                         const tString &servername, const tOutput &score,
+                         const tOutput &users     , const tOutput &ping)
 {
+    tColoredString highlight, normal;
+    highlight << tColoredString::ColorString( 1,.7,.7 );
+    normal << tColoredString::ColorString( .7,.3,.3 );
+
     tString sn, s, u, p;
-    sn << servername;
+
+    sn << normal;
+    s << normal;
+    u << normal;
+    p << normal;
+
+    switch ( sortKey_ )
+    {
+    case nServerInfo::KEY_NAME:
+        sn = highlight;
+        break;
+    case nServerInfo::KEY_PING:
+        p = highlight;
+        break;
+    case nServerInfo::KEY_USERS:
+        u = highlight;
+        break;
+    case nServerInfo::KEY_SCORE:
+        s = highlight;
+        break;
+    case nServerInfo::KEY_MAX:
+        break;
+    }
+
+    sn << servername;// tColoredString::RemoveColors( servername );
     s  << score;
     u  << users;
     p  << ping;
-    ::Render(y, sn, s, u, p);
+
+    Render(y, sn, s, u, p);
 }
 
-#endif
+#endif /* DEDICATED */
+static bool sg_filterServernameColorStrings = true;
+static tSettingItem< bool > removeServerNameColors("FILTER_COLOR_SERVER_NAMES", sg_filterServernameColorStrings);
+static bool sg_filterServernameDarkColorStrings = true;
+static tSettingItem< bool > removeServerNameDarkColors("FILTER_DARK_COLOR_SERVER_NAMES", sg_filterServernameDarkColorStrings);
 
 void gServerMenuItem::Render(REAL x,REAL y,REAL alpha, bool selected)
 {
 #ifndef DEDICATED
-    REAL time=tSysTimeFloat()*10;
+    // REAL time=tSysTimeFloat()*10;
 
-    if(selected)
-        glColor4f(.8+.2*sin(time),.3-.1*sin(time),.3-.1*sin(time),alpha);
-    else
-        glColor4f(1,1,1,alpha);
+    SetColor( selected, alpha );
 
+    gServerMenu *serverMenu = static_cast<gServerMenu*>(menu);
 
     if (server)
     {
+        tColoredString name;
         tString score;
         tString users;
         tString ping;
@@ -466,26 +643,41 @@ void gServerMenuItem::Render(REAL x,REAL y,REAL alpha, bool selected)
         else if ( server->Users() >= server->MaxUsers() )
         {
             score << tOutput( "$network_master_full" );
+            score << " (" << server->Users() << "/" << server->MaxUsers() << ")";
         }
         else
         {
+            if ( favorite_ )
+            {
+                score << "B ";
+            }
+
             score << s;
-            users << server->Users();
+            users << server->Users() << "/" << server->MaxUsers();
             ping  << p;
         }
 
-        ::Render(y*shrink + displace,
-                 server->Name(),
-                 score, users, ping);
+        if ( sg_filterServernameColorStrings )
+            name << tColoredString::RemoveColors( server->GetName(), false );
+	else if ( sg_filterServernameDarkColorStrings )
+            name << tColoredString::RemoveColors( server->GetName(), true );
+        else
+        {
+            name << server->GetName();
+        }
+
+        serverMenu->Render(y*shrink + displace,
+                           name,
+                           score, users, ping);
     }
     else
     {
         tOutput o("$network_master_noserver");
         tString s;
         s << o;
-        ::Render(y*shrink + displace,
-                 s,
-                 "", "", "");
+        serverMenu->Render(y*shrink + displace,
+                           s,
+                           tString(""), tString(""), tString(""));
 
     }
 #endif
@@ -499,45 +691,146 @@ void gServerMenuItem::RenderBackground()
 
     if ( server )
     {
-        glColor3f(1,1,1);
+        rTextField::SetDefaultColor( tColor(1,1,1) );
 
         rTextField players( -.9, -.3, text_width, text_height );
         players << tOutput( "$network_master_players" );
         if ( server->UserNamesOneLine().Len() > 2 )
-            players << RemoveColors(server->UserNamesOneLine());
+            players << server->UserNamesOneLine();
         else
             players << tOutput( "$network_master_players_empty" );
+        players << "\n" << tColoredString::ColorString(1,1,1);
+        tColoredString uri;
+        uri << server->Url() << tColoredString::ColorString(1,1,1);
+        players << tOutput( "$network_master_serverinfo", server->Release(), uri, server->Options() );
     }
 #endif
 }
 
+#ifndef DEDICATED
+static void Refresh()
+{
+    continuePoll = true;
+    nServerInfo::StartQueryAll( sg_queryType );
+}
+#endif
+
+bool gBrowserMenuItem::Event( SDL_Event& event )
+{
+#ifndef DEDICATED
+    switch (event.type)
+    {
+    case SDL_KEYDOWN:
+        switch (event.key.keysym.sym)
+        {
+        case SDLK_r:
+            {
+                static double lastRefresh = - 100; //!< the time of the last manual refresh
+                if ( tSysTimeFloat() - lastRefresh > 2.0 )
+                {
+                    lastRefresh = tSysTimeFloat();
+                    // trigger refresh
+                    st_ToDo( Refresh );
+                    return true;
+                }
+            }
+            break;
+        default:
+            break;
+        }
+    }
+#endif
+
+    return uMenuItem::Event( event );
+}
+
+
+bool gServerMenuItem::Event( SDL_Event& event )
+{
+#ifndef DEDICATED
+    switch (event.type)
+    {
+    case SDL_KEYDOWN:
+        switch (event.key.keysym.sym)
+        {
+        case SDLK_p:
+            continuePoll = true;
+            if ( server && tSysTimeFloat() - lastPing_ > .5f )
+            {
+                lastPing_ = tSysTimeFloat();
+
+                server->SetQueryType( nServerInfo::QUERY_ALL );
+                server->QueryServer();
+                server->ClearInfoFlags();
+            }
+            return true;
+            break;
+        default:
+            break;
+        }
+        switch (event.key.keysym.unicode)
+        {
+        case '+':
+            if ( server )
+            {
+                server->SetScoreBias( server->GetScoreBias() + 10 );
+                server->CalcScore();
+            }
+            (static_cast<gServerMenu*>(menu))->Update();
+
+            return true;
+            break;
+        case '-':
+            if ( server )
+            {
+                server->SetScoreBias( server->GetScoreBias() - 10 );
+                server->CalcScore();
+            }
+            (static_cast<gServerMenu*>(menu))->Update();
+
+            return true;
+            break;
+        case 'b':
+            if ( server && !favorite_ )
+            {
+                favorite_ = gServerFavorites::AddFavorite( server );
+            }
+            return true;
+            break;
+        default:
+            break;
+        }
+    }
+#endif
+
+    return gBrowserMenuItem::Event( event );
+}
+
 void gBrowserMenuItem::RenderBackground()
 {
     sn_Receive();
+    sn_SendPlanned();
 
     menu->GenericBackground();
     if (continuePoll)
     {
         continuePoll = nServerInfo::DoQueryAll(sg_simultaneous);
         sn_Receive();
-    }
-
-    static REAL timeout=-1E+32f;
-
-    if (timeout < tSysTimeFloat())
-    {
-        (static_cast<gServerMenu*>(menu))->Update();
-        timeout = tSysTimeFloat()+2.0f;
+        sn_SendPlanned();
     }
 
 #ifndef DEDICATED
-    glColor4f(.8,.3,.3,1);
+    rTextField::SetDefaultColor( tColor(.8,.3,.3,1) );
 
-    ::Render(.62,
-             tOutput("$network_master_servername"),
-             tOutput("$network_master_score"),
-             tOutput("$network_master_users"),
-             tOutput("$network_master_ping"));
+	tString sn2 = tString(tOutput("$network_master_servername"));
+	if (getFriendsEnabled()) //display that friends filter is on
+		sn2 << " - " << tOutput("$friends_enable");
+
+    static_cast<gServerMenu*>(menu)->Render(.62,
+                                            sn2,
+                                            tOutput("$network_master_score"),
+                                            tOutput("$network_master_users"),
+                                            tOutput("$network_master_ping"));
 #endif
 }
 
@@ -573,6 +866,8 @@ void gServerMenuItem::SetServer(nServerInfo *s)
 
         server->menuItem = this;
     }
+
+    favorite_ = gServerFavorites::IsFavorite( server );
 }
 
 gServerInfo *gServerMenuItem::GetServer()
@@ -580,10 +875,10 @@ gServerInfo *gServerMenuItem::GetServer()
     return server;
 }
 
-static char *sg_HelpText = "$network_master_browserhelp";
+static char const * sg_HelpText = "$network_master_browserhelp";
 
 gServerMenuItem::gServerMenuItem(gServerMenu *men)
-        :gBrowserMenuItem(men, sg_HelpText), server(NULL)
+        :gBrowserMenuItem(men, sg_HelpText), server(NULL), lastPing_(-100), favorite_(false)
 {}
 
 gServerMenuItem::~gServerMenuItem()
@@ -609,17 +904,15 @@ gServerInfo::~gServerInfo()
 void gServerStartMenuItem::Render(REAL x,REAL y,REAL alpha, bool selected)
 {
 #ifndef DEDICATED
-    REAL time=tSysTimeFloat()*10;
+    // REAL time=tSysTimeFloat()*10;
 
-    if(selected)
-        glColor4f(.8+.2*sin(time),.3-.1*sin(time),.3-.1*sin(time),alpha);
-    else
-        glColor4f(1,1,1,alpha);
+    SetColor( selected, alpha );
 
-
-    ::Render(y*shrink + displace,
-             tOutput("$network_master_start"),
-             "", "", "");
+    tString s;
+    s << tOutput("$network_master_start");
+    static_cast<gServerMenu*>(menu)->Render(y*shrink + displace,
+                                            s,
+                                            tString(), tString(), tString());
 #endif
 }
 
